@@ -1,9 +1,10 @@
+from __future__ import annotations
+
 import os
 import re
-import subprocess
 import webbrowser
 from datetime import datetime, timedelta
-from typing import Optional
+from subprocess import PIPE, CompletedProcess
 
 import click
 from click import secho
@@ -14,8 +15,10 @@ from delfino.models import AppContext
 from delfino.terminal_output import print_header
 from delfino.validation import assert_package_manager_is_known, assert_pip_package_installed
 
-from delfino_core.commands.verify_all import verify_all
+from delfino_core.commands.verify import run_group_verify
 from delfino_core.config import CorePluginConfig
+from delfino_core.spinner import Spinner
+from delfino_core.utils import ask
 
 try:
     from git import Repo
@@ -23,13 +26,14 @@ except ImportError:
     pass
 
 
-def _run(args: str) -> subprocess.CompletedProcess:
+def _run(args: str, spinner: Spinner | None = None) -> CompletedProcess:
     """Print the command before execution."""
-    return run(args, on_error=OnError.EXIT, capture_output=True)
+    if spinner is None:
+        return run(args, on_error=OnError.EXIT, stdin=PIPE, stderr=PIPE)
 
-
-def ask(question: str) -> bool:
-    return bool(input(f"\033[1;33m{question} [Y/n]: \033[0m").lower() == "y")
+    result = run(args, on_error=OnError.PASS, running_hook=spinner, stdout=PIPE, stderr=PIPE)
+    spinner.print_results(result, error_cls=click.exceptions.Exit)
+    return result
 
 
 class Updater:
@@ -38,15 +42,16 @@ class Updater:
     @staticmethod
     def _git_root():
         return (
-            run(["git", "rev-parse", "--show-toplevel"], capture_output=True, on_error=OnError.EXIT)
+            run(["git", "rev-parse", "--show-toplevel"], stdin=PIPE, stderr=PIPE, on_error=OnError.EXIT)
             .stdout.decode()
             .strip()
         )
 
-    def __init__(self):
+    def __init__(self, stash: bool):
         assert_pip_package_installed("gitpython")
 
         self._repo = Repo(self._git_root())
+        self._stash = stash
         now = datetime.utcnow()
         self._start_of_week = now - timedelta(now.isoweekday() - 1)
 
@@ -54,7 +59,7 @@ class Updater:
         commit_message = f"Dependencies rollup: {self._start_of_week.strftime('%Y-%m-%d')}"
         can_update = self._repo.commit("HEAD").message.strip() == commit_message
 
-        if ask("Do you want to commit changes now?"):
+        if do_commit := ask("Do you want to commit changes now?"):
             if can_update:  # update existing commit
                 if self._repo.is_dirty():
                     _run("git pull")
@@ -63,7 +68,7 @@ class Updater:
                 _run("git add .")
                 _run(f"git commit -a -m '{commit_message}'")
 
-        if ask("Do you want to push changes now?"):
+        if do_commit and ask("Do you want to push changes now?"):
             if can_update:
                 _run("git push --force-with-lease")
             else:
@@ -76,10 +81,10 @@ class Updater:
                 else:
                     secho(f"\nOpen a new pull request by visiting:\n\n\t{url}\n", fg="green")
 
-    def _link_to_open_a_pull_request(self) -> Optional[str]:
+    def _link_to_open_a_pull_request(self) -> str | None:
         url = self._repo.remote().url
-        match = re.match("git@github.com:(.*)\\.git", url)
-        if not match:
+
+        if not (match := re.match("git@github.com:(.*)\\.git", url)):
             match = re.match("https://github.com/(.*)\\.git", url)
 
         return f"https://github.com/{match.group(1)}/pull/new/{self._repo.active_branch}" if match else None
@@ -123,8 +128,9 @@ class Updater:
             secho(f"Branch '{branch}' already exists.", fg="yellow")
             _run(f"git checkout {branch}")
         else:
-            secho("Stashing existing changes.", fg="yellow")
-            _run("git stash")
+            if self._stash:
+                secho("Stashing existing changes.", fg="yellow")
+                _run("git stash")
 
             if self._repo.active_branch != "main":
                 secho("Checking out 'main'.", fg="yellow")
@@ -168,19 +174,18 @@ class PipenvUpdater(Updater):
         _run("pipenv sync -d")
 
     def print_outdated_packages_and_lock_if_changed(self) -> bool:
-        secho("Updating packages based on version pinning. This will take a while ...", fg="yellow")
-        _run("pipenv update -d")
+        spinner = Spinner("pipenv", "updating packages based on version pinning")
+        _run("pipenv update -d", spinner)
 
-        secho("Checking outdated packages. This will take a while ...", fg="yellow")
-        result = _run("pipenv update --outdated")
+        spinner = Spinner("pipenv", "checking outdated packages")
+        result = _run("pipenv update --outdated", spinner)
 
         pipfile = self._read_dependency_file()
 
         available_updates = []
 
         for line in result.stdout.decode().split(os.linesep) + result.stderr.decode().split(os.linesep):
-            match = re.match(self._SKIP_PATTERN, line) or re.match(self._OUTDATED_PATTEN, line)
-            if not match:
+            if not (match := re.match(self._SKIP_PATTERN, line) or re.match(self._OUTDATED_PATTEN, line)):
                 continue
 
             package = match.group("package")
@@ -203,13 +208,11 @@ class PoetryUpdater(Updater):
     _FILENAME = "pyproject.toml"
 
     def print_outdated_packages_and_lock_if_changed(self) -> bool:
-        secho("Updating packages based on version pinning. This will take a while ...", fg="yellow")
-        _run("poetry update")
+        spinner = Spinner("poetry", "updating packages based on version pinning")
+        _run("poetry update", spinner)
 
-        secho("Checking outdated packages. This will take a while ...", fg="yellow")
-        result = _run("poetry show --outdated --why --ansi")
-
-        if not result.stdout:
+        spinner = Spinner("poetry", "checking outdated packages")
+        if not (result := _run("poetry show --outdated --why --ansi", spinner)).stdout:
             return False
 
         pyproject_toml = self._read_dependency_file()
@@ -219,11 +222,20 @@ class PoetryUpdater(Updater):
         return self._read_dependency_file() != pyproject_toml
 
 
-@click.command()
-@click.option("--retry", default=False, show_default=True, is_flag=True, help=" Retry an update after failed tests.")
+@click.command("dependencies-update")
+@click.option("--retry", default=False, show_default=True, is_flag=True, help="Retry an update after failed tests.")
+@click.option(
+    "--stash/--no-stash",
+    default=True,
+    show_default=True,
+    help=(
+        "Don't stash existing changes. Useful when there are some existing manual changes "
+        "that should be also part of the upgrade."
+    ),
+)
 @pass_app_context(CorePluginConfig)
 @click.pass_context
-def dependencies_update(click_context: click.Context, app_context: AppContext, retry):
+def run_dependencies_update(click_context: click.Context, app_context: AppContext, retry: bool, stash: bool):
     """Manages the process of updating dependencies."""
     print_header("Updating dependencies", icon="🔄")
 
@@ -231,9 +243,9 @@ def dependencies_update(click_context: click.Context, app_context: AppContext, r
     updater: Updater
 
     if app_context.package_manager == PackageManager.PIPENV:
-        updater = PipenvUpdater()
+        updater = PipenvUpdater(stash)
     elif app_context.package_manager == PackageManager.POETRY:
-        updater = PoetryUpdater()
+        updater = PoetryUpdater(stash)
     else:
         raise AssertionError(
             f"The '{app_context.package_manager.value}' package manager is not supported by this command."
@@ -242,7 +254,7 @@ def dependencies_update(click_context: click.Context, app_context: AppContext, r
     updater.update(retry)
 
     try:
-        click_context.invoke(verify_all)
+        click_context.invoke(run_group_verify)
     except Exception:
         secho(
             f"\nOne or more checks have failed. Fix any issues above and then run:\n\n\t"
