@@ -11,19 +11,23 @@ import click
 from delfino.decorators import files_folders_option, pass_args
 from delfino.execution import OnError, run
 from delfino.models import AppContext
-from delfino.terminal_output import print_header, print_no_issues_found
 from delfino.validation import assert_pip_package_installed, pip_package_installed
 
 from delfino_core.backports import path_is_relative_to
 from delfino_core.config import CorePluginConfig, pass_plugin_app_context
-from delfino_core.utils import execute_commands_group
+from delfino_core.spinner import Spinner
+from delfino_core.utils import commands_group_help, execute_commands_group
 
 
-@click.command()
+@click.command("pydocstyle")
 @pass_args
 @files_folders_option
 @pass_plugin_app_context
-def lint_pydocstyle(app_context: AppContext[CorePluginConfig], passed_args: Tuple[str, ...], files_folders: Tuple[str]):
+def run_pydocstyle(
+    app_context: AppContext[CorePluginConfig],
+    passed_args: Tuple[str, ...],
+    files_folders: Tuple[str],
+):
     """Run docstring linting on source code.
 
     Docstring linting is done via pydocstyle. The pydocstyle config can be found in the
@@ -33,25 +37,43 @@ def lint_pydocstyle(app_context: AppContext[CorePluginConfig], passed_args: Tupl
     """
     assert_pip_package_installed("pydocstyle")
 
-    print_header("documentation style", level=2)
+    spinner = Spinner("pydocstyle", "checking documentation style")
     dirs = build_target_paths(app_context, files_folders, False, False)
 
-    run(
+    results = run(
         ["pydocstyle", *passed_args, *dirs],
-        stdout=PIPE,
-        on_error=OnError.ABORT,
+        on_error=OnError.PASS,
         env_update_path={"PYTHONPATH": app_context.plugin_config.sources_directory},
+        stdout=PIPE,
+        stderr=PIPE,
+        running_hook=spinner,
     )
 
-    print_no_issues_found()
+    spinner.print_results(results)
 
 
-@click.command()
+@click.command("ruff")
 @pass_args
 @files_folders_option
 @pass_plugin_app_context
-def lint_pycodestyle(
-    app_context: AppContext[CorePluginConfig], passed_args: Tuple[str, ...], files_folders: Tuple[str]
+def run_ruff(app_context: AppContext[CorePluginConfig], passed_args: Tuple[str, ...], files_folders: Tuple[str]):
+    """Run ruff."""
+    assert_pip_package_installed("ruff")
+
+    dirs = build_target_paths(app_context, files_folders)
+    spinner = Spinner("ruff", "checking code")
+    results = run(["ruff", *passed_args, *dirs], stdout=PIPE, stderr=PIPE, on_error=OnError.PASS, running_hook=spinner)
+    spinner.print_results(results)
+
+
+@click.command("pycodestyle")
+@pass_args
+@files_folders_option
+@pass_plugin_app_context
+def run_pycodestyle(
+    app_context: AppContext[CorePluginConfig],
+    passed_args: Tuple[str, ...],
+    files_folders: Tuple[str],
 ):
     """Run PEP8 checking on code.
 
@@ -62,8 +84,6 @@ def lint_pycodestyle(
     """
     assert_pip_package_installed("pycodestyle")
 
-    print_header("code style (PEP8)", level=2)
-
     dirs = build_target_paths(app_context, files_folders)
 
     # TODO(Radek): Implement unofficial config support in pyproject.toml by parsing it
@@ -75,49 +95,32 @@ def lint_pycodestyle(
         "pycodestyle",
         "--ignore",
         "E501,W503,E231,E203,E402",
+        # Ignores explained:
+        # - E501: Line length is checked by PyLint
+        # - W503: Disable checking of "Line break before binary operator". PEP8 recently (~2019) switched to
+        #         "line break before the operator" style, so we should permit this usage.
+        # - E231: "missing whitespace after ','" is a false positive. Handled by black formatter.
         "--exclude",
         ".svn,CVS,.bzr,.hg,.git,__pycache__,.tox,*_config_parser.py",
         *passed_args,
         *dirs,
     ]
-    run(
+    spinner = Spinner("pycodestyle", "checking code style")
+    result = run(
         args,
         stdout=PIPE,
-        on_error=OnError.ABORT,
+        stderr=PIPE,
+        on_error=OnError.PASS,
         env_update_path={"PYTHONPATH": app_context.plugin_config.sources_directory},
+        running_hook=spinner,
     )
-    # Ignores explained:
-    # - E501: Line length is checked by PyLint
-    # - W503: Disable checking of "Line break before binary operator". PEP8 recently (~2019) switched to
-    #         "line break before the operator" style, so we should permit this usage.
-    # - E231: "missing whitespace after ','" is a false positive. Handled by black formatter.
-
-    print_no_issues_found()
-
-
-def run_pylint(
-    sources_dir: Path,
-    checked_dirs: List[Path],
-    pylintrc_folder: Path,
-    passed_args: Tuple[str, ...],
-):
-    print_header(", ".join(map(str, checked_dirs)), level=3)
-
-    run(
-        ["pylint", "-j", str(cpu_count()), "--rcfile", pylintrc_folder / ".pylintrc", *passed_args, *checked_dirs],
-        stdout=PIPE,
-        on_error=OnError.ABORT,
-        env_update_path={"PYTHONPATH": sources_dir},
-    )
-
-    print_no_issues_found()
+    spinner.print_results(result)
 
 
 @lru_cache(maxsize=1)
 def cpu_count():
     if getenv("CI", ""):
-        cpu_shares = Path("/sys/fs/cgroup/cpu/cpu.shares")
-        if cpu_shares.is_file():
+        if (cpu_shares := Path("/sys/fs/cgroup/cpu/cpu.shares")).is_file():
             return int(cpu_shares.read_text(encoding="utf-8").strip()) // 1024
 
     log = logging.getLogger("cpu_count")
@@ -126,8 +129,7 @@ def cpu_count():
     if pip_package_installed("psutil"):
         import psutil  # pylint: disable=import-outside-toplevel
 
-        count = psutil.cpu_count(logical=False)
-        if not count:
+        if not (count := psutil.cpu_count(logical=False)):
             log.warning(fallback_msg)
             return 1
         return count
@@ -144,22 +146,27 @@ def build_target_paths(
 ) -> List[Path]:
     if files_folders:
         return [Path(path) for path in files_folders]
-    local_commands_directory = app_context.pyproject_toml.tool.delfino.local_commands_directory
     plugin_config = app_context.plugin_config
     target_paths: List[Path] = [plugin_config.sources_directory]
 
     if include_tests and plugin_config.tests_directory.exists():
         target_paths.append(plugin_config.tests_directory)
-    if include_commands and local_commands_directory.exists():
-        target_paths.append(local_commands_directory)
+    if include_commands:
+        target_paths.extend(
+            folder for folder in app_context.pyproject_toml.tool.delfino.local_command_folders if folder.exists()
+        )
     return target_paths
 
 
-@click.command()
+@click.command("pylint")
 @files_folders_option
 @pass_args
 @pass_plugin_app_context
-def lint_pylint(app_context: AppContext[CorePluginConfig], passed_args: Tuple[str, ...], files_folders: Tuple[str]):
+def run_pylint(
+    app_context: AppContext[CorePluginConfig],
+    passed_args: Tuple[str, ...],
+    files_folders: Tuple[str],
+):
     """Run pylint on code.
 
     The bulk of our code conventions are enforced via pylint. The pylint config can be
@@ -167,7 +174,6 @@ def lint_pylint(app_context: AppContext[CorePluginConfig], passed_args: Tuple[st
     """
     assert_pip_package_installed("pylint")
 
-    print_header("pylint", level=2)
     plugin_config = app_context.plugin_config
 
     def get_pylintrc_folder(path: Path) -> Path:
@@ -178,17 +184,35 @@ def lint_pylint(app_context: AppContext[CorePluginConfig], passed_args: Tuple[st
     target_paths = build_target_paths(app_context, files_folders)
     grouped_paths = groupby(target_paths, get_pylintrc_folder)
     for pylintrc_folder, paths in grouped_paths:
-        run_pylint(app_context.plugin_config.sources_directory, list(paths), pylintrc_folder, passed_args)
+        checked_dirs = list(paths)
+        spinner = Spinner("pylint", f"checking '{', '.join(map(str, checked_dirs))}'")
+        results = run(
+            [
+                "pylint",
+                "-j",
+                str(cpu_count()),
+                "--rcfile",
+                pylintrc_folder / ".pylintrc",
+                *passed_args,
+                *checked_dirs,
+            ],
+            stdout=PIPE,
+            stderr=PIPE,
+            on_error=OnError.PASS,  # spinner aborts on error
+            env_update_path={"PYTHONPATH": app_context.plugin_config.sources_directory},
+            running_hook=spinner,
+        )
+        spinner.print_results(results)
 
 
-@click.command(help="Runs all linting commands. Configured by the ``lint_commands`` setting.")
+@click.command("lint", help=commands_group_help("lint"))
 @files_folders_option
 @pass_plugin_app_context
 @click.pass_context
-def lint(
+def run_group_lint(
     click_context: click.Context,
     app_context: AppContext[CorePluginConfig],
     **kwargs,
 ):
     del kwargs  # passed to downstream commands
-    execute_commands_group("lint", click_context, app_context.plugin_config)
+    execute_commands_group(click_context, app_context.plugin_config)
