@@ -16,8 +16,10 @@ class CorePluginConfig(PluginConfig):
     pytest_modules: List[str] = Field(default_factory=list)
     reports_directory: Path = Path("reports")
     test_types: List[str] = ["unit", "integration"]
-    verify_commands: Tuple[str, ...] = ("format", "lint", "typecheck", "test-all")
-    lint_commands: Tuple[str, ...] = ("lint-pylint", "lint-pycodestyle", "lint-pydocstyle")
+    verify_commands: Tuple[str, ...] = ("format", "lint", "mypy", "test")
+    lint_commands: Tuple[str, ...] = ("ruff", "pylint", "pycodestyle", "pydocstyle")
+    format_commands: Tuple[str, ...] = ("ensure-pre-commit", "pyupgrade", "isort", "black")
+    test_commands: Tuple[str, ...] = ("pytest", "coverage-report")
     disable_pre_commit: bool = False
     typecheck: Typecheck = Field(default_factory=Typecheck)
 
